﻿using AircraftPlantContracts.BindingModels;
using AircraftPlantContracts.BusinessLogicsContracts;
using AircraftPlantContracts.SearchModels;
using AircraftPlantContracts.StoragesContracts;
using AircraftPlantContracts.ViewModels;
using Microsoft.Extensions.Logging;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace AircraftPlantBusinessLogic.BusinessLogics
{
    /// <summary>
    /// Реализация интерфейса бизнес-логики для исполнителей
    /// </summary>
    public class ImplementerLogic : IImplementerLogic
    {
        /// <summary>
        /// Логгер
        /// </summary>
        private readonly ILogger _logger;

        /// <summary>
        /// Взаимодействие с хранилищем исполнителей
        /// </summary>
        private readonly IImplementerStorage _implementerStorage;

        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="logger"></param>
        /// <param name="implemeneterStorage"></param>
        public ImplementerLogic(ILogger<ImplementerLogic> logger, IImplementerStorage implementerStorage)
        {
            _logger = logger;
            _implementerStorage = implementerStorage;
        }

        /// <summary>
        /// Получение списка
        /// </summary>
        /// <param name="model"></param>
        /// <returns></returns>
        public List<ImplementerViewModel>? ReadList(ImplementerSearchModel? model)
        {
            _logger.LogInformation("ReadList. ImplementerFIO:{ImplementerFIO}.Id:{Id}", model?.ImplementerFIO, model?.Id);
            var list = model == null ? _implementerStorage.GetFullList() : _implementerStorage.GetFilteredList(model);
            
            if (list == null)
            {
                _logger.LogWarning("ReadList return null list");
                return null;
            }

            _logger.LogInformation("ReadList. Count:{Count}", list.Count);
            return list;
        }

        /// <summary>
        /// Получение отдельной записи
        /// </summary>
        /// <param name="model"></param>
        /// <returns></returns>
        /// <exception cref="ArgumentNullException"></exception>
        public ImplementerViewModel? ReadElement(ImplementerSearchModel model)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }

            _logger.LogInformation("ReadElement. ImplementerFIO:{ImplementerFIO}.Id:{Id}", model.ImplementerFIO, model.Id);
            
            var element = _implementerStorage.GetElement(model);
            if (element == null)
            {
                _logger.LogWarning("ReadElement element not found");
                return null;
            }

            _logger.LogInformation("ReadElement find. Id:{Id}", element.Id);
            return element;
        }

        /// <summary>
        /// Создание записи
        /// </summary>
        /// <param name="model"></param>
        /// <returns></returns>
        public bool Create(ImplementerBindingModel model)
        {
            CheckModel(model);

            if (_implementerStorage.Insert(model) == null)
            {
                _logger.LogWarning("Insert operation failed");
                return false;
            }
            return true;
        }

        /// <summary>
        /// Изменение записи
        /// </summary>
        /// <param name="model"></param>
        /// <returns></returns>
        public bool Update(ImplementerBindingModel model)
        {
            CheckModel(model);

            if (_implementerStorage.Update(model) == null)
            {
                _logger.LogWarning("Update operation failed");
                return false;
            }
            return true;
        }

        /// <summary>
        /// Удаление записи
        /// </summary>
        /// <param name="model"></param>
        /// <returns></returns>
        public bool Delete(ImplementerBindingModel model)
        {
            CheckModel(model, false);
            _logger.LogInformation("Delete. Id:{Id}", model.Id);
            
            if (_implementerStorage.Delete(model) == null)
            {
                _logger.LogWarning("Delete operation failed");
                return false;
            }
            return true;
        }

        private void CheckModel(ImplementerBindingModel model, bool withParams = true)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            if (!withParams)
            {
                return;
            }
            if (string.IsNullOrEmpty(model.ImplementerFIO))
            {
                throw new ArgumentNullException("У исполнителя должно быть ФИО", nameof(model.ImplementerFIO));
            }
            if (string.IsNullOrEmpty(model.Password))
            {
                throw new ArgumentNullException("У исполнителя должен быть пароль", nameof(model.Password));
            }
            if (model.WorkExperience <= 0)
            {
                throw new ArgumentNullException("У исполнителя стаж должен быть больше 0", nameof(model.WorkExperience));
            }
            if (model.Qualification <= 0)
            {
                throw new ArgumentNullException("У исполнителя квалификация должна быть больше 0", nameof(model.Qualification));
            }
            
            _logger.LogInformation("Implementer. ImplementerFIO:{ImplementerFIO}.Qualification:{Qualification}.WorkExperience:{WorkExperience} Id: {Id}", model.ImplementerFIO, model.Qualification, model.WorkExperience, model.Id);
            var element = _implementerStorage.GetElement(new ImplementerSearchModel
            {
                ImplementerFIO = model.ImplementerFIO
            });
            if (element != null && element.Id != model.Id)
            {
                throw new InvalidOperationException("Исполнитель с таким ФИО уже есть");
            }
        }
    }
}
