﻿using AircraftPlantContracts.BindingModels;
using AircraftPlantContracts.BusinessLogicsContracts;
using AircraftPlantContracts.SearchModels;
using AircraftPlantContracts.StoragesContracts;
using AircraftPlantContracts.ViewModels;
using AircraftPlantDataModels.Enums;
using AircraftPlantDataModels.Models;
using Microsoft.Extensions.Logging;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace AircraftPlantBusinessLogic.BusinessLogics
{
    /// <summary>
    /// Реализация интерфейса бизнес-логики для заказов
    /// </summary>
    public class OrderLogic : IOrderLogic
    {
        /// <summary>
        /// Логгер
        /// </summary>
        private readonly ILogger _logger;

        /// <summary>
        /// Взаимодействие с хранилищем заказов
        /// </summary>
        private readonly IOrderStorage _orderStorage;

        /// <summary>
        /// Взаимодействие с хранилищем магазинов
        /// </summary>
        private IShopStorage _shopStorage;

        /// <summary>
        /// Бизнес-логика магазинов
        /// </summary>
        private IShopLogic _shopLogic;

        /// <summary>
        /// Взаимодействие с хранилищем изделий
        /// </summary>
        private IPlaneStorage _planeStorage;

        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="logger"></param>
        /// <param name="orderStorage"></param>
        public OrderLogic(ILogger<OrderLogic> logger, IOrderStorage orderStorage, IShopStorage shopStorage, IShopLogic shopLogic, IPlaneStorage planeStorage)
        {
            _logger = logger;
            _orderStorage = orderStorage;
            _shopStorage = shopStorage;
            _shopLogic = shopLogic;
            _planeStorage = planeStorage;
        }

        /// <summary>
        /// Получение списка
        /// </summary>
        /// <param name="model"></param>
        /// <returns></returns>
        public List<OrderViewModel>? ReadList(OrderSearchModel? model)
        {
            _logger.LogInformation("ReadList. Order.Id:{ Id}", model?.Id);

            var list = model == null ? _orderStorage.GetFullList() : _orderStorage.GetFilteredList(model);
            if (list == null)
            {
                _logger.LogWarning("ReadList return null list");
                return null;
            }

            _logger.LogInformation("ReadList. Count:{Count}", list.Count);
            return list;
        }

        /// <summary>
        /// Получение отдельной записи
        /// </summary>
        /// <param name="model"></param>
        /// <returns></returns>
        /// <exception cref="ArgumentNullException"></exception>
        public OrderViewModel? ReadElement(OrderSearchModel model)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            _logger.LogInformation("ReadElement. Id:{ Id}", model.Id);

            var element = _orderStorage.GetElement(model);
            if (element == null)
            {
                _logger.LogWarning("ReadElement element not found");
                return null;
            }

            _logger.LogInformation("ReadElement find. Id:{Id}", element.Id);
            return element;
        }

        /// <summary>
        /// Создание заказа
        /// </summary>
        /// <param name="model"></param>
        /// <returns></returns>
        public bool CreateOrder(OrderBindingModel model)
        {
            CheckModel(model);
            if (model.Status != OrderStatus.Неизвестен)
            {
                _logger.LogWarning("Insert operation failed. Order status incorrect.");
                return false;
            }

            model.Status = OrderStatus.Принят;

            if (_orderStorage.Insert(model) == null)
            {
                _logger.LogWarning("Insert operation failed");
                return false;
            }
            return true;
        }

        /// <summary>
        /// Смена статуса заказа (Выполняется)
        /// </summary>
        /// <param name="model"></param>
        /// <returns></returns>
        public bool TakeOrderInWork(OrderBindingModel model)
        {
            return StatusUpdate(model, OrderStatus.Выполняется);
        }

        /// <summary>
        /// Смена статуса заказа (Выдан)
        /// </summary>
        /// <param name="model"></param>
        /// <returns></returns>
        public bool FinishOrder(OrderBindingModel model)
        {
            return StatusUpdate(model, OrderStatus.Выдан);
        }

        /// <summary>
        /// Смена статуса заказа (Готов)
        /// </summary>
        /// <param name="model"></param>
        /// <returns></returns>
        public bool DeliveryOrder(OrderBindingModel model)
        {
            return StatusUpdate(model, OrderStatus.Готов);
        }

        /// <summary>
        /// Проверка модели заказа
        /// </summary>
        /// <param name="model"></param>
        /// <param name="withParams"></param>
        private void CheckModel(OrderBindingModel model, bool withParams = true)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            if (!withParams)
            {
                return;
            }
            if (model.PlaneId < 0)
            {
                throw new ArgumentNullException("Некорректный идентификатор изделия", nameof(model.PlaneId));
            }
            if (model.Count <= 0)
            {
                throw new ArgumentNullException("Количество изделий в заказе должно быть больше 0", nameof(model.Count));
            }
            if (model.Sum <= 0)
            {
                throw new ArgumentNullException("Сумма заказа должна быть больше 0", nameof(model.Sum));
            }
            _logger.LogInformation("Order. OrderID:{Id}.Sum:{ Sum}. PlaneId: { PlaneId}", model.Id, model.Sum, model.PlaneId);
        }

        /// <summary>
        /// Смена статуса заказа
        /// </summary>
        /// <param name="model"></param>
        /// <param name="newStatus"></param>
        /// <returns></returns>
        private bool StatusUpdate(OrderBindingModel model, OrderStatus newStatus)
        {
            var element = _orderStorage.GetElement(new OrderSearchModel
            {
                Id = model.Id
            });
            if (element == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            if (element.Status + 1 != newStatus && element.Status != OrderStatus.Ожидание)
            {
                _logger.LogWarning("Change status operation failed");
                return false;
            }

            model.Status = newStatus;

            if (model.Status == OrderStatus.Готов || element.Status == OrderStatus.Ожидание)
            {
                var plane = _planeStorage.GetElement(new PlaneSearchModel { Id = element.PlaneId });
                if (plane == null)
                {
                    _logger.LogWarning("Status change error. Plane not found");
                    return false;
                }

                if (!CheckSupply(plane, element.Count))
                {
                    model.Status = OrderStatus.Ожидание;
                    _logger.LogWarning("Status change error. Shop is overflowed");
                }
                else
                {
                    model.DateImplement = DateTime.Now;
                }
            }
            else
            {
                model.DateImplement = element.DateImplement;
            }

            CheckModel(model, false);
            if (_orderStorage.Update(model) == null)
            {
                _logger.LogWarning("Change status operation failed");
                return false;
            }
            return true;
        }

        /// <summary>
        /// Проверка заказа
        /// </summary>
        /// <param name="plane"></param>
        /// <param name="count"></param>
        /// <returns></returns>
        public bool CheckSupply(IPlaneModel model, int count)
        {
            if (count <= 0)
            {
                _logger.LogWarning("Check supply operation error. Planes count < 0");
                return false;
            }

            int sumCapacity = _shopStorage.GetFullList().Select(x => x.MaxPlanes).Sum();
            int sumCount = _shopStorage.GetFullList().Select(x => x.ShopPlanes.Select(y => y.Value.Item2).Sum()).Sum();
            int free = sumCapacity - sumCount;
            if (free < count)
            {
                _logger.LogWarning("Check supply error. No place for new planes");
                return false;
            }

            foreach (var shop in _shopStorage.GetFullList())
            {
                free = shop.MaxPlanes;
                foreach (var plane in shop.ShopPlanes)
                {
                    free -= plane.Value.Item2;
                }

                if (free == 0)
                {
                    continue;
                }

                if (free >= count)
                {
                    if (_shopLogic.AddPlaneInShop(new()
                    {
                        Id = shop.Id
                    }, model, count))
                    {
                        count = 0;
                    }
                    else
                    {
                        _logger.LogWarning("Supply error");
                        return false;
                    }
                }
                else
                {
                    if (_shopLogic.AddPlaneInShop(new()
                    {
                        Id = shop.Id
                    }, model, free))
                    {
                        count -= free;
                    }
                    else
                    {
                        _logger.LogWarning("Supply error");
                        return false;
                    }
                }
                if (count <= 0)
                {
                    return true;
                }
            }
            return false;
        }
    }
}
