﻿using AircraftPlantContracts.BindingModels;
using AircraftPlantContracts.BusinessLogicsContracts;
using AircraftPlantContracts.SearchModels;
using AircraftPlantContracts.StoragesContracts;
using AircraftPlantContracts.ViewModels;
using AircraftPlantDataModels.Models;
using Microsoft.Extensions.Logging;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace AircraftPlantBusinessLogic.BusinessLogics
{
    /// <summary>
    /// Реализация интерфейса бизнес-логики для магазинов
    /// </summary>
    public class ShopLogic : IShopLogic
    {
        /// <summary>
        /// Логгер
        /// </summary>
        private readonly ILogger _logger;

        /// <summary>
        /// Взаимодействие с хранилищем магазинов
        /// </summary>
        private readonly IShopStorage _shopStorage;

        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="logger"></param>
        /// <param name="shopStorage"></param>
        public ShopLogic(ILogger<ShopLogic> logger, IShopStorage shopStorage)
        {
            _logger = logger;
            _shopStorage = shopStorage;
        }

        /// <summary>
        /// Получение списка
        /// </summary>
        /// <param name="model"></param>
        /// <returns></returns>
        public List<ShopViewModel>? ReadList(ShopSearchModel? model)
        {
            _logger.LogInformation("ReadList. ShopName:{ShopName}.Id:{ Id}", model?.ShopName, model?.Id);
            
            var list = model == null ? _shopStorage.GetFullList() : _shopStorage.GetFilteredList(model);
            if (list == null)
            {
                _logger.LogWarning("ReadList return null list");
                return null;
            }

            _logger.LogInformation("ReadList. Count:{Count}", list.Count);
            return list;
        }

        /// <summary>
        /// Получение отдельной записи
        /// </summary>
        /// <param name="model"></param>
        /// <returns></returns>
        /// <exception cref="ArgumentNullException"></exception>
        public ShopViewModel? ReadElement(ShopSearchModel model)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }

            _logger.LogInformation("ReadElement. ShopName:{ShopName}.Id:{ Id}", model.ShopName, model.Id);
            
            var element = _shopStorage.GetElement(model);
            if (element == null)
            {
                _logger.LogWarning("ReadElement element not found");
                return null;
            }

            _logger.LogInformation("ReadElement find. Id:{Id}", element.Id);
            return element;
        }

        /// <summary>
        /// Создание записи
        /// </summary>
        /// <param name="model"></param>
        /// <returns></returns>
        public bool Create(ShopBindingModel model)
        {
            CheckModel(model);

            if (_shopStorage.Insert(model) == null)
            {
                _logger.LogWarning("Insert operation failed");
                return false;
            }
            return true;
        }

        /// <summary>
        /// Изменение записи
        /// </summary>
        /// <param name="model"></param>
        /// <returns></returns>
        public bool Update(ShopBindingModel model)
        {
            CheckModel(model);

            if (_shopStorage.Update(model) == null)
            {
                _logger.LogWarning("Update operation failed");
                return false;
            }
            return true;
        }

        /// <summary>
        /// Удаление записи
        /// </summary>
        /// <param name="model"></param>
        /// <returns></returns>
        public bool Delete(ShopBindingModel model)
        {
            CheckModel(model, false);
            _logger.LogInformation("Delete. Id:{Id}", model.Id);

            if (_shopStorage.Delete(model) == null)
            {
                _logger.LogWarning("Delete operation failed");
                return false;
            }
            return true;
        }

        /// <summary>
        /// Добавление изделия в магазин
        /// </summary>
        /// <param name="model"></param>
        /// <param name="plane"></param>
        /// <param name="count"></param>
        /// <returns></returns>
        /// <exception cref="ArgumentNullException"></exception>
        /// <exception cref="ArgumentException"></exception>
        public bool AddPlaneInShop(ShopSearchModel model, IPlaneModel plane, int count)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            if (count <= 0)
            {
                throw new ArgumentException("Количество изделий должно быть больше 0", nameof(count));
            }
            _logger.LogInformation("AddPlaneInShop. ShopName:{ShopName}.Id:{ Id}", model.ShopName, model.Id);
            var element = _shopStorage.GetElement(model);
            if (element == null)
            {
                _logger.LogWarning("AddPlaneInShop element not found");
                return false;
            }
            _logger.LogInformation("AddPlaneInShop find. Id:{Id}", element.Id);

            var countPlanes = element.ShopPlanes.Select(x => x.Value.Item2).Sum();
            if (element.MaxPlanes - countPlanes < count)
            {
                _logger.LogWarning("Shop is overflowed");
                return false;
            }

            if (element.ShopPlanes.TryGetValue(plane.Id, out var pair))
            {
                element.ShopPlanes[plane.Id] = (plane, count + pair.Item2);
                _logger.LogInformation("AddPlaneInShop. Added {count} {plane} to '{ShopName}' shop", count, plane.PlaneName, element.ShopName);
            }
            else
            {
                element.ShopPlanes[plane.Id] = (plane, count);
                _logger.LogInformation("AddPlaneInShop. Added {count} new plane {plane} to '{ShopName}' shop", count, plane.PlaneName, element.ShopName);
            }

            _shopStorage.Update(new()
            {
                Id = element.Id,
                Address = element.Address,
                ShopName = element.ShopName,
                DateOpening = element.DateOpening,
                ShopPlanes = element.ShopPlanes,
                MaxPlanes = element.MaxPlanes
            });
            return true;
        }

        /// <summary>
        /// Продажа изделий
        /// </summary>
        /// <param name="plane"></param>
        /// <param name="count"></param>
        /// <returns></returns>
        /// <exception cref="ArgumentNullException"></exception>
        /// <exception cref="ArgumentException"></exception>
        public bool SellPlanes(IPlaneModel plane, int count)
        {
            if (plane == null)
            {
                throw new ArgumentNullException(nameof(plane));
            }
            if (count <= 0)
            {
                throw new ArgumentException("Количество изделий должно быть больше 0", nameof(count));
            }

            if (_shopStorage.SellPlanes(plane, count))
            {
                _logger.LogInformation("Selling sucsess");
                return true;
            }
            _logger.LogInformation("Selling failed");
            return false;
        }

        /// <summary>
        /// Проверка модели магазина
        /// </summary>
        /// <param name="model"></param>
        /// <param name="withParams"></param>
        /// <exception cref="ArgumentNullException"></exception>
        /// <exception cref="InvalidOperationException"></exception>
        private void CheckModel(ShopBindingModel model, bool withParams = true)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            if (!withParams)
            {
                return;
            }
            if (string.IsNullOrEmpty(model.ShopName))
            {
                throw new ArgumentNullException("Нет названия магазина", nameof(model.ShopName));
            }
            _logger.LogInformation("Shop. ShopName:{ShopName}.Address:{ Address}. Id:{ Id}", model.ShopName, model.Address, model.Id);
            var element = _shopStorage.GetElement(new ShopSearchModel
            {
                ShopName = model.ShopName
            });
            if (element != null && element.Id != model.Id && element.ShopName == model.ShopName)
            {
                throw new InvalidOperationException("Магазин с таким названием уже есть");
            }
        }
    }
}
