﻿using AircraftPlantContracts.BindingModels;
using AircraftPlantContracts.SearchModels;
using AircraftPlantContracts.StoragesContracts;
using AircraftPlantContracts.ViewModels;
using AircraftPlantDatabaseImplement.Models;
using Microsoft.EntityFrameworkCore;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace AircraftPlantDatabaseImplement.Implements
{
    /// <summary>
    /// Реализация интерфейса хранилища для заказов
    /// </summary>
    public class OrderStorage : IOrderStorage
    {
        /// <summary>
        /// Получение полного списка
        /// </summary>
        /// <returns></returns>
        public List<OrderViewModel> GetFullList()
        {
            using var context = new AircraftPlantDatabase();
            return context.Orders
                .Include(x => x.Plane)
                .Include(x => x.Client)
                .Include(x => x.Implementer)
                .Select(x => x.GetViewModel)
                .ToList();
        }

        /// <summary>
        /// Получение фильтрованного списка
        /// </summary>
        /// <param name="model"></param>
        /// <returns></returns>
        public List<OrderViewModel> GetFilteredList(OrderSearchModel model)
        {
            if (!model.Id.HasValue && !model.ClientId.HasValue && (!model.DateFrom.HasValue || !model.DateTo.HasValue) && model.Status == null)
            {
                return new();
            }

            using var context = new AircraftPlantDatabase();

            if (model.DateFrom.HasValue)
            {
                return context.Orders
                    .Include(x => x.Plane)
                    .Include(x => x.Client)
                    .Include(x => x.Implementer)
                    .Where(x => x.DateCreate >= model.DateFrom && x.DateCreate <= model.DateTo)
                    .Select(x => x.GetViewModel)
                    .ToList();
            }

            if (model.ClientId.HasValue)
            {
                return context.Orders
                    .Include(x => x.Plane)
                    .Include(x => x.Client)
                    .Include(x => x.Implementer)
                    .Where(x => x.ClientId == model.ClientId)
                    .Select(x => x.GetViewModel)
                    .ToList();
            }

            if (model.Status.HasValue)
            {
                return context.Orders
                        .Include(x => x.Plane)
                        .Include(x => x.Client)
                        .Include(x => x.Implementer)
                        .Where(x => x.Status == model.Status)
                        .Select(x => x.GetViewModel)
                        .ToList();
            }

            return context.Orders
                .Include(x => x.Plane)
                .Include(x => x.Client)
                .Where(x => x.Id.Equals(model.Id))
                .Select(x => x.GetViewModel)
                .ToList();
        }

        /// <summary>
        /// Получение элемента
        /// </summary>
        /// <param name="model"></param>
        /// <returns></returns>
        public OrderViewModel? GetElement(OrderSearchModel model)
        {
            if (!model.Id.HasValue)
            {
                return null;
            }

            using var context = new AircraftPlantDatabase();
            if (model.Id.HasValue)
            {
                return context.Orders
                    .Include(x => x.Plane)
                    .Include(x => x.Client)
                    .Include(x => x.Implementer)
                    .FirstOrDefault(x => x.Id == model.Id)
                    ?.GetViewModel;
            }

            if (model.ImplementerId.HasValue && model.Status.HasValue)
            {
                return context.Orders
                    .Include(x => x.Plane)
                    .Include(x => x.Client)
                    .Include(x => x.Implementer)
                    .FirstOrDefault(x => x.ImplementerId == model.ImplementerId && x.Status == model.Status)
                    ?.GetViewModel;
            }

            if (model.ImplementerId.HasValue)
            {
                return context.Orders
                    .Include(x => x.Plane)
                    .Include(x => x.Client)
                    .Include(x => x.Implementer)
                    .FirstOrDefault(x => x.ImplementerId == model.ImplementerId)
                    ?.GetViewModel;
            }

            return null;
        }

        /// <summary>
        /// Добавление элемента
        /// </summary>
        /// <param name="model"></param>
        /// <returns></returns>
        public OrderViewModel? Insert(OrderBindingModel model)
        {
            using var context = new AircraftPlantDatabase();
            var newOrder = Order.Create(context, model);
            if (newOrder == null)
            {
                return null;
            }

            context.Orders.Add(newOrder);
            context.SaveChanges();
            return newOrder.GetViewModel;
        }

        /// <summary>
        /// Редактирование элемента
        /// </summary>
        /// <param name="model"></param>
        /// <returns></returns>
        public OrderViewModel? Update(OrderBindingModel model)
        {
            using var context = new AircraftPlantDatabase();
            var order = context.Orders.FirstOrDefault(x  => x.Id == model.Id);
            if (order == null)
            {
                return null;
            }

            order.Update(model);
            context.SaveChanges();
            return order.GetViewModel;
        }

        /// <summary>
        /// Удаление элемента
        /// </summary>
        /// <param name="model"></param>
        /// <returns></returns>
        public OrderViewModel? Delete(OrderBindingModel model)
        {
            using var context = new AircraftPlantDatabase();
            var element = context.Orders.FirstOrDefault(x => x.Id == model.Id);
            if (element != null)
            {
                context.Orders.Remove(element);
                context.SaveChanges();
                return element.GetViewModel;
            }
            return null;
        }
    }
}
