﻿using AircraftPlantContracts.BindingModels;
using AircraftPlantContracts.SearchModels;
using AircraftPlantContracts.StoragesContracts;
using AircraftPlantContracts.ViewModels;
using AircraftPlantDatabaseImplement.Models;
using AircraftPlantDataModels.Models;
using Microsoft.EntityFrameworkCore;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace AircraftPlantDatabaseImplement.Implements
{
    /// <summary>
    /// Реализация интерфейса хранилища для магазинов
    /// </summary>
    public class ShopStorage : IShopStorage
    {
        /// <summary>
        /// Получение полного списка
        /// </summary>
        /// <returns></returns>
        public List<ShopViewModel> GetFullList()
        {
            using var context = new AircraftPlantDatabase();
            return context.Shops
                .Include(x => x.Planes)
                .ThenInclude(x => x.Plane)
                .ToList()
                .Select(x => x.GetViewModel)
                .ToList();
        }

        /// <summary>
        /// Получение фильтрованного списка
        /// </summary>
        /// <param name="model"></param>
        /// <returns></returns>
        public List<ShopViewModel> GetFilteredList(ShopSearchModel model)
        {
            if (string.IsNullOrEmpty(model.ShopName))
            {
                return new();
            }

            using var context = new AircraftPlantDatabase();
            return context.Shops
                .Include(x => x.Planes)
                .ThenInclude(x => x.Plane)
                .Where(x => x.ShopName.Contains(model.ShopName))
                .ToList()
                .Select(x => x.GetViewModel)
                .ToList();
        }

        /// <summary>
        /// Получение элемента
        /// </summary>
        /// <param name="model"></param>
        /// <returns></returns>
        public ShopViewModel? GetElement(ShopSearchModel model)
        {
            if (string.IsNullOrEmpty(model.ShopName) && !model.Id.HasValue)
            {
                return null;
            }

            using var context = new AircraftPlantDatabase();
            return context.Shops
                .Include(x => x.Planes)
                .ThenInclude(x => x.Plane)
                .FirstOrDefault(x => (!string.IsNullOrEmpty(model.ShopName) &&
                    x.ShopName == model.ShopName) ||
                    (model.Id.HasValue && x.Id == model.Id))
                ?.GetViewModel;
        }

        /// <summary>
        /// Добавление элемента
        /// </summary>
        /// <param name="model"></param>
        /// <returns></returns>
        public ShopViewModel? Insert(ShopBindingModel model)
        {
            using var context = new AircraftPlantDatabase();
            var newShop = Shop.Create(context, model);
            if (newShop == null)
            {
                return null;
            }

            context.Shops.Add(newShop);
            context.SaveChanges();
            return newShop.GetViewModel;
        }

        /// <summary>
        /// Редактирование элемента
        /// </summary>
        /// <param name="model"></param>
        /// <returns></returns>
        public ShopViewModel? Update(ShopBindingModel model)
        {
            using var context = new AircraftPlantDatabase();
            using var transaction = context.Database.BeginTransaction();
            try
            {
                var shop = context.Shops.FirstOrDefault(rec => rec.Id == model.Id);
                if (shop == null)
                {
                    return null;
                }

                shop.Update(model);
                context.SaveChanges();
                shop.UpdatePlanes(context, model);
                transaction.Commit();
                return shop.GetViewModel;
            }
            catch
            {
                transaction.Rollback();
                throw;
            }
        }

        /// <summary>
        /// Удаление элемента
        /// </summary>
        /// <param name="model"></param>
        /// <returns></returns>
        public ShopViewModel? Delete(ShopBindingModel model)
        {
            using var context = new AircraftPlantDatabase();
            var element = context.Shops
                .Include(x => x.Planes)
                .FirstOrDefault(rec => rec.Id == model.Id);
            if (element != null)
            {
                context.Shops.Remove(element);
                context.SaveChanges();
                return element.GetViewModel;
            }
            return null;
        }

        /// <summary>
        /// Продажа изделий
        /// </summary>
        /// <param name="model"></param>
        /// <param name="count"></param>
        /// <returns></returns>
        public bool SellPlanes(IPlaneModel model, int count)
        {
            using var context = new AircraftPlantDatabase();
            using var transaction = context.Database.BeginTransaction();
            try
            {
                var shops = context.Shops
                    .Include(x => x.Planes)
                    .ThenInclude(x => x.Plane)
                    .ToList()
                    .Where(x => x.ShopPlanes.ContainsKey(model.Id));

                foreach (var shop in shops)
                {
                    int countInCurrentShop = shop.ShopPlanes[model.Id].Item2;
                    if (countInCurrentShop <= count)
                    {
                        var elem = context.ShopPlanes
                            .Where(x => x.PlaneId == model.Id)
                            .FirstOrDefault(x => x.ShopId == shop.Id);
                        context.ShopPlanes.Remove(elem);
                        shop.ShopPlanes.Remove(model.Id);
                        count -= countInCurrentShop;
                    }
                    else
                    {
                        shop.ShopPlanes[model.Id] = (shop.ShopPlanes[model.Id].Item1, countInCurrentShop - count);
                        count = 0;
                        shop.UpdatePlanes(context, new ShopBindingModel
                        {
                            Id = shop.Id,
                            ShopName = shop.ShopName,
                            Address = shop.Address,
                            DateOpening = shop.DateOpening,
                            ShopPlanes = shop.ShopPlanes,
                            MaxPlanes = shop.MaxPlanes
                        });
                    }
                    if (count <= 0)
                    {
                        context.SaveChanges();
                        transaction.Commit();
                        return true;
                    }
                }
                transaction.Rollback();
                return false;
            }
            catch
            {
                transaction.Rollback();
                throw;
            }
        }

        /// <summary>
        /// Проверка наличия в нужном количестве
        /// </summary>
        /// <param name="model"></param>
        /// <param name="count"></param>
        /// <returns></returns>
        /// <exception cref="NotImplementedException"></exception>
        public bool CheckCount(IPlaneModel model, int count)
        {
            throw new NotImplementedException();
        }
    }
}
