﻿using AircraftPlantContracts.BindingModels;
using AircraftPlantContracts.ViewModels;
using AircraftPlantDataModels.Enums;
using AircraftPlantDataModels.Models;
using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace AircraftPlantDatabaseImplement.Models
{
    /// <summary>
    /// Сущность "Заказ"
    /// </summary>
    public class Order : IOrderModel
    {
        /// <summary>
        /// Идентификатор
        /// </summary>
        public int Id { get; set; }

        /// <summary>
        /// Идентификатор изделия
        /// </summary>
        [Required]
        public int PlaneId { get; set; }

        /// <summary>
        /// Сущность "Изделие"
        /// </summary>
        public virtual Plane Plane { get; set; } = new();

        /// <summary>
        /// Идентификатор клиента
        /// </summary>
        [Required]
        public int ClientId { get; set; }

        /// <summary>
        /// Сущность "Клиент"
        /// </summary>
        public Client Client { get; set; } = new();

        /// <summary>
        /// Идентификатор исполнителя
        /// </summary>
        public int? ImplementerId { get; set; }

        /// <summary>
        /// Сущность "Исполнитель"
        /// </summary>
        public virtual Implementer? Implementer { get; set; } = new();

        /// <summary>
        /// Количество изделий
        /// </summary>
        [Required]
        public int Count { get; set; }

        /// <summary>
        /// Сумма заказа
        /// </summary>
        [Required]
        public double Sum { get; set; }

        /// <summary>
        /// Статус заказа
        /// </summary>
        [Required]
        public OrderStatus Status { get; set; }

        /// <summary>
        /// Дата создания заказа
        /// </summary>
        [Required]
        public DateTime DateCreate { get; set; }

        /// <summary>
        /// Дата выполнения заказа
        /// </summary>
        public DateTime? DateImplement { get; set; }

        /// <summary>
        /// Создание модели заказа
        /// </summary>
        /// <param name="model"></param>
        /// <returns></returns>
        public static Order? Create(AircraftPlantDatabase context, OrderBindingModel? model)
        {
            if (model == null)
            {
                return null;
            }

            return new Order
            {
                Id = model.Id,
                PlaneId = model.PlaneId,
                Plane = context.Planes.First(x => x.Id == model.PlaneId),
                ClientId = model.ClientId,
                Client = context.Clients.First(x => x.Id == model.ClientId),
                ImplementerId = model.ImplementerId,
                Implementer = model.ImplementerId.HasValue ? context.Implementers.First(x => x.Id == model.ImplementerId) : null,
                Count = model.Count,
                Sum = model.Sum,
                Status = model.Status,
                DateCreate = model.DateCreate,
                DateImplement = model.DateImplement
            };
        }

        /// <summary>
        /// Изменение модели заказа
        /// </summary>
        /// <param name="model"></param>
        public void Update(OrderBindingModel? model)
        {
            if (model == null)
            {
                return;
            }

            Status = model.Status;
            DateImplement = model.DateImplement;
            if (model.ImplementerId.HasValue)
                ImplementerId = model.ImplementerId;
        }

        /// <summary>
        /// Получение модели заказа
        /// </summary>
        public OrderViewModel GetViewModel => new()
        {
            Id = Id,
            PlaneId = PlaneId,
            PlaneName = Plane.PlaneName,
            ClientId = ClientId,
            ClientFIO = Client.ClientFIO,
            ImplementerId = ImplementerId,
            ImplementerFIO = Implementer?.ImplementerFIO ?? string.Empty,
            Count = Count,
            Sum = Sum,
            Status = Status,
            DateCreate = DateCreate,
            DateImplement = DateImplement
        };
    }
}
