﻿using AircraftPlantContracts.BindingModels;
using AircraftPlantContracts.SearchModels;
using AircraftPlantContracts.StoragesContracts;
using AircraftPlantContracts.ViewModels;
using AircraftPlantFileImplement.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace AircraftPlantFileImplement.Implements
{
    /// <summary>
    /// Реализация интерфейса хранилища для клиентов
    /// </summary>
    public class ClientStorage : IClientStorage
    {
        /// <summary>
        /// Хранилище
        /// </summary>
        private readonly DataFileSingleton _source;

        /// <summary>
        /// Конструктор
        /// </summary>
        public ClientStorage()
        {
            _source = DataFileSingleton.GetInstance();
        }

        /// <summary>
        /// Получение полного списка
        /// </summary>
        /// <returns></returns>
        public List<ClientViewModel> GetFullList()
        {
            return _source.Clients
                .Select(x => x.GetViewModel)
                .ToList();
        }

        /// <summary>
        /// Получение фильтрованного списка
        /// </summary>
        /// <param name="model"></param>
        /// <returns></returns>
        public List<ClientViewModel> GetFilteredList(ClientSearchModel model)
        {
            if (string.IsNullOrEmpty(model.Email))
            {
                return new();
            }

            return _source.Clients
                .Where(x => x.Email.Contains(model.Email))
                .Select(x => x.GetViewModel)
                .ToList();
        }

        /// <summary>
        /// Получение элемента
        /// </summary>
        /// <param name="model"></param>
        /// <returns></returns>
        public ClientViewModel? GetElement(ClientSearchModel model)
        {
            if (string.IsNullOrEmpty(model.Email) && !model.Id.HasValue)
            {
                return null;
            }

            return _source.Clients
                .FirstOrDefault(x =>
                    (!string.IsNullOrEmpty(model.Email) &&
                    x.Email == model.Email) ||
                    (model.Id.HasValue && x.Id == model.Id))
                ?.GetViewModel;
        }

        /// <summary>
        /// Добавление элемента
        /// </summary>
        /// <param name="model"></param>
        /// <returns></returns>
        public ClientViewModel? Insert(ClientBindingModel model)
        {
            model.Id = _source.Clients.Count > 0 ? _source.Clients.Max(x => x.Id) + 1 : 1;

            var newClient = Client.Create(model);
            if (newClient == null)
            {
                return null;
            }

            _source.Clients.Add(newClient);
            _source.SaveClients();
            return newClient.GetViewModel;
        }

        /// <summary>
        /// Редактирование элемента
        /// </summary>
        /// <param name="model"></param>
        /// <returns></returns>
        public ClientViewModel? Update(ClientBindingModel model)
        {
            var client = _source.Clients.FirstOrDefault(x => x.Id == model.Id);
            if (client == null)
            {
                return null;
            }

            client.Update(model);
            _source.SaveClients();
            return client.GetViewModel;
        }

        /// <summary>
        /// Удаление элемента
        /// </summary>
        /// <param name="model"></param>
        /// <returns></returns>
        public ClientViewModel? Delete(ClientBindingModel model)
        {
            var element = _source.Clients.FirstOrDefault(x => x.Id == model.Id);
            if (element != null)
            {
                _source.Clients.Remove(element);
                _source.SaveClients();
                return element.GetViewModel;
            }
            return null;
        }
    }
}
